// ͻ
//  Modulname     Datum       Version   Programmierer  
// Ķ
//  HPGLVIEW.CPP  07.05.1998  V 1.0     A. Meile, 6 Ia 
// ͹
//  Zweck  Einfaches Anzeigeprogramm fr HPGL-Dateien    
//         (bung im Fach Compilerbau), Hauptprogramm    
// ͹
//  nderungen und berarbeitungen                        
// Ķ
// Version Datum      Programmierer                    
// Ķ
//  V -.-  --.--.---- -. ------      --                
// ͼ

#include <stdio.h>
#include "lieshpgl.hpp"     // Parser-Hilfspaket fr Read-Ahead-Lesen
#include "plothpgl.hpp"     // Software-Plotter fr Ausgabe

LiesHpgl *s;          // HPGL-Strom
PlotHpgl *p;          // Software-Plotter

int ignUnsup = 0,     // Flag, um unbekanntes ignorieren
    ignFalsch = 0;    // Flag, um fehlerhafte Kommandos ignorieren

void gibFehlerUndHilfeAus(const char *pgNa) {
  fprintf(stderr, "Ungltige Parameter\n"
  "\n"
  "Aufrufformat: %s [-i|I] [-rXRxYR] [-fAFar] [-mSpeiGr] <Dateiname>\n"
  "\n"
  "Optionen:\n"
  "-i        nicht untersttzte HPGL-Befehle einfach berlesen\n"
  "-I        alle fehlerhaften und nicht unterst. Kommandos ignorieren\n"
  "-rXRxYR   Bildschirmauflsung whlen (Vorgabe -r640x480)\n"
  "-fAFar    Anzahl Bildschirmfarben whlen (Vorgabe -f16)\n"
  "-mSpeiGr Grsse des Zeichenpunktspeichers (Vorgae -m10000)\n", pgNa);
  exit(1);
}

void keinRestZul(const char bef[]) {
  char puf[50];
  if(ignFalsch || s->aktuellesZeichen() == ';')
    s->ueberliesRest();  // alles o.k. => berlesen
  else {
    sprintf(puf, "unzulssiger Rest nach %s-Befehl", bef);
    s->gibFehlerAus(puf);
  }
}
void nichtUnterst(char k1, char k2) {
  char puf[50];
  if(ignUnsup)
    s->ueberliesRest();   // stillschweigend ignorieren
  else {
    sprintf(puf, "Nicht untersttztes Kommando %c%c", k1, k2);
    s->gibFehlerAus(puf);
  }
}

// Unbekanntes Kommando
void falsch(char k1, char k2) {
  char puf[50];
  if(ignFalsch)
    s->ueberliesRest();   // stillschweigend ignorieren
  else {
    sprintf(puf, "unbekanntes Kommando %c%c", k1, k2);
    s->gibFehlerAus(puf);
  }
}

// Hauptprogramm

int main(int argc, char *argv[]) {
  char *dName;          // Dateiname

  int xAufl = 640,      // Horizontale Auflsung
      yAufl = 480,      // Vertikale Auflsung
      aFar = 16,        // Anzahl Farben
      speiGroe = 10000, // Speichergrsse Plotter
      argWied = 0,      // fr Argumentwiederholungstest
      i,                // Hilfszhlvariable
      wert1, wert2;     // Werteparameter

  
  // Kommandozeilenargumente auswerten
  for(i=1; i<argc; i++) {
    if(argv[i][0] == '-')    // Option vorliegend
      switch(argv[i][1]) {
        case 'i' :           // Unimplementiertes ignorieren
          if(strlen(argv[i]) != 2 || argWied & 0x1) 
            gibFehlerUndHilfeAus(argv[0]);
          else {
            ignUnsup = 1;
            argWied |= 0x1;
          }
        break;
        case 'I' :           // Alle Fehler ignorieren
          if(strlen(argv[i]) != 2 || argWied & 0x1)
            gibFehlerUndHilfeAus(argv[0]);
          else {
            ignUnsup = 1;
            ignFalsch = 1;
            argWied |= 0x1;
          }
        break;
        case 'r' :           // Auflsung whlen
          if(sscanf(argv[i], "-r%dx%d", &xAufl, &yAufl) != 2 ||
          xAufl < 1 || yAufl < 1 || argWied & 0x2)
            gibFehlerUndHilfeAus(argv[0]);
          else
            argWied |= 0x2;
        break;
        case 'f' :           // Farbtiefe whlen
          if(sscanf(argv[i], "-f%d", &aFar) != 1 || aFar < 1 ||
          argWied & 0x4)
            gibFehlerUndHilfeAus(argv[0]);
          else
            argWied |= 0x4;
        break;
        case 'm' :           // Speichergrsse whlen
          if(sscanf(argv[i], "-m%d", &speiGroe) != 1 || speiGroe < 1 ||
          argWied & 0x8)
            gibFehlerUndHilfeAus(argv[0]);
          else
            argWied |= 0x8;
        break;
        default:      // ungltige Option
          gibFehlerUndHilfeAus(argv[0]);
      }
    else if(argWied & 0x10)
      gibFehlerUndHilfeAus(argv[0]);
    else { // Dateiname
      dName = argv[i];
      argWied |= 0x10;
    }
  }
  if(!(argWied & 0x10))        // kein Dateiname
    gibFehlerUndHilfeAus(argv[0]);

  s = new LiesHpgl(dName);     // Strom erffnen
  p = new PlotHpgl(speiGroe);  // Plotter anlegen

  // ab hier eigentlicher Parser
  while(!s->endeErreicht()) {
    switch(s->aktuellesZeichen()) {
      case '\033' :   // RS232-Kommandos:
        s->liesEinZeichen();
        switch(s->aktuellesZeichen()) {
          case '@' :  // Plotterkonfiguration setzen
          case 'A' :  // Ausgabe identifizieren
          case 'B' :  // Ausgabepuffergrsse
          case 'E' :  // Erweiterter Ausgabefehler
          case 'H' :  // Handshakemodus 1
          case 'I' :  // Handshakemodus 2
          case 'J' :  // Gerteabbruch
          case 'K' :  // Grafikabbruch
          case 'L' :  // Ausgabepuffergrsse, wenn leer
          case 'M' :  // Ausgabemodus setzen
          case 'N' :  // Erweiterte Ausgabe und Handshake setzen
          case 'O' :  // Erweiterter Ausgabezustand
          case 'P' :  // Handshakemodus setzen
          case 'Q' :  // Monitormodus
          case 'R' :  // Reset
          case 'S' :  // Konfigurierbarer Speicher ausgeben
          case 'T' :  // Konfigurierbarer Speicher reservieren
          case 'U' :  // Erw. Ausgabemodus beenden
          case 'Y' :  // Plotter ein
          case '(' :  // Plotter ein
          case 'Z' :  // Plotter aus
          case ')' :  // Plotter aus
            nichtUnterst('[', s->aktuellesZeichen());
          break;
          default  :
            falsch('[', s->aktuellesZeichen());
        }
      break;
      case 'A' :
        s->liesEinZeichen();
        switch(s->aktuellesZeichen()) {
          case 'A' :  // Absolutbogen
          case 'F' :  // Blatteinzug
          case 'H' :  // Blatteinzug
          case 'P' :  // Autom. Stiftoperationen
          case 'R' :  // Relativbogen
          case 'S' :  // Beschleunigungsauswahl
            nichtUnterst('A', s->aktuellesZeichen());
          break;
          default  :
            falsch('A', s->aktuellesZeichen());
        }
      break;
      case 'B' :
        s->liesEinZeichen();
        switch(s->aktuellesZeichen()) {
          case 'F' :  // Plot puffern
          case 'L' :  // Schriftzug puffern
            nichtUnterst('B', s->aktuellesZeichen());
          break;
          default  :
            falsch('B', s->aktuellesZeichen());
        }
      break;
      case 'C' :
        s->liesEinZeichen();
        switch(s->aktuellesZeichen()) {
          case 'A' :  // Alternativzeichensatz
          case 'C' :  // Zeichensatzwinkel
          case 'I' :  // Kreis
          case 'M' :  // Zeichenwahlmodus
          case 'P' :  // Zeichen plotten
          case 'S' :  // Standardzeichensatz
          case 'T' :  // Kurventoleranz
          case 'V' :  // Kurvenliniengenerator
            nichtUnterst('C', s->aktuellesZeichen());
          break;
          default  :
            falsch('C', s->aktuellesZeichen());
        }
      break;
      case 'D' :
        s->liesEinZeichen();
        switch(s->aktuellesZeichen()) {
          case 'C' :  // Digitalisieren lschen
          case 'F' :  // Vorgabe
          case 'I' :  // Absolute Richtung
          case 'L' :  // Herunterladezeichen
          case 'P' :  // Plot digitalisieren
          case 'R' :  // Relative Richtung
          case 'S' :  // Zeichen in Platz
          case 'T' :  // Textendzeichen setzen
            nichtUnterst('D', s->aktuellesZeichen());
          break;
          default  :
            falsch('D', s->aktuellesZeichen());
        }
      break;
      case 'E' :
        s->liesEinZeichen();
        switch(s->aktuellesZeichen()) {
          case 'A' :  // Absolutes Kantenrechteck
          case 'P' :  // Kantenpolygon
          case 'R' :  // Relatives Kantenrechteck
          case 'S' :  // Extra Leerraum
          case 'W' :  // Kantenkeil
            nichtUnterst('E', s->aktuellesZeichen());
          break;
          default  :
            falsch('E', s->aktuellesZeichen());
        }
      break;
      case 'F' :
        s->liesEinZeichen();
        switch(s->aktuellesZeichen()) {
          case 'P' :  // Polygon fllen
          case 'S' :  // Auswahl erzwingen
          case 'T' :  // Fllart
            nichtUnterst('F', s->aktuellesZeichen());
          break;
          default  :
            falsch('F', s->aktuellesZeichen());
        }
      break;
      case 'G' :
        s->liesEinZeichen();
        switch(s->aktuellesZeichen()) {
          case 'C' :  // Gruppenzhler
          case 'M' :  // Grafikspeicher
            nichtUnterst('G', s->aktuellesZeichen());
          break;
          default  :
            falsch('G', s->aktuellesZeichen());
        }
      break;
      case 'I' :
        s->liesEinZeichen();
        switch(s->aktuellesZeichen()) {
          case 'N' :  // Plotterreset
            s->liesEinZeichen();
            keinRestZul("IN");    // keine Resten tolerieren
          break;
          case 'M' :  // Eingabemaske
          case 'P' :  // P1 und P2 eingeben
          case 'V' :  // Zeichensatzspeicher aufrufen
          case 'W' :  // Eingabefenster
            nichtUnterst('I', s->aktuellesZeichen());
          break;
          default  :
            falsch('I', s->aktuellesZeichen());
        }
      break;
      case 'K' :
        s->liesEinZeichen();
        switch(s->aktuellesZeichen()) {
          case 'Y' :  // Schlssel definieren
            nichtUnterst('K', s->aktuellesZeichen());
          break;
          default  :
            falsch('K', s->aktuellesZeichen());
        }
      break;
      case 'L' :
        s->liesEinZeichen();
        switch(s->aktuellesZeichen()) {
          case 'B' :  // Text
          case 'O' :  // Textursprung
          case 'T' :  // Linientyp
            nichtUnterst('L', s->aktuellesZeichen());
          break;
          default  :
            falsch('L', s->aktuellesZeichen());
        }
      break;
      case 'N' :
        s->liesEinZeichen();
        switch(s->aktuellesZeichen()) {
          case 'R' :  // Nicht bereit (Blattauswurf)
            nichtUnterst('N', s->aktuellesZeichen());
          break;
          default  :
            falsch('N', s->aktuellesZeichen());
        }
      break;
      case 'O' :
        s->liesEinZeichen();
        switch(s->aktuellesZeichen()) {
          case 'A' :  // eigentliche Position und Stiftzustand ausgeben
          case 'C' :  // Kommando- und Stiftzustand ausgeben
          case 'D' :  // Digitalisierter Punkt und Stiftzustand ausgeben
          case 'E' :  // Fehler ausgeben
          case 'F' :  // Faktoren ausgeben
          case 'G' :  // Anzahl Gruppen ausgeben
          case 'H' :  // Hardwaremssige Begrenzungen ausgeben
          case 'I' :  // Identifikation ausgeben
          case 'K' :  // Schlssel ausgeben
          case 'L' :  // Textlnge ausgeben
          case 'O' :  // Einstellungen ausgeben
          case 'P' :  // P1 und P2 ausgeben
          case 'S' :  // Zustand ausgeben
          case 'T' :  // Stiftkaruselltyp ausgeben
          case 'W' :  // Fenster ausgeben
            nichtUnterst('O', s->aktuellesZeichen());
          break;
          default  :
            falsch('O', s->aktuellesZeichen());
        }
      break;
      case 'P' :
        s->liesEinZeichen();
        switch(s->aktuellesZeichen()) {
          case 'A' :  // Absolut plotten
            s->liesEinZeichen();
            if(s->liesZahl(wert1) || s->liesBestimmtesZeichen(',') ||
            s->liesZahl(wert2)) {
              if(ignFalsch)
                s->ueberliesRest();
              else
                s->gibFehlerAus("PA: Parameter X,Y erwartet");
            }
            keinRestZul("PA");
            p->plotAbs(wert1, wert2);   // Plotten
          break;
          case 'R' :  // Relativ plotten
            s->liesEinZeichen();
            if(s->liesZahl(wert1) || s->liesBestimmtesZeichen(',') ||
            s->liesZahl(wert2)) {
              if(ignFalsch)
                s->ueberliesRest();
              else
                s->gibFehlerAus("PR: Parameter X,Y erwartet");
            }
            keinRestZul("PR");
            p->plotRel(wert1, wert2);   // Plotten
          break;
          case 'D' :  // Stift senken
            s->liesEinZeichen();
            keinRestZul("PD");
            p->stiftUnten();
          break;
          case 'U' :  // Stift heben
            s->liesEinZeichen();
            keinRestZul("PU");
            p->stiftOben();
          break;
          case 'B' :  // Gepufferter Text plotten
          case 'G' :  // Blatt einziehen
          case 'T' :  // Stiftstrke
            nichtUnterst('D', s->aktuellesZeichen());
          break;
          default  :
            falsch('D', s->aktuellesZeichen());
        }
      break;
      case 'R' :
        s->liesEinZeichen();
        switch(s->aktuellesZeichen()) {
          case 'A' :  // Rechteck fllen absolut
          case 'O' :  // Koordinatensystem drehen
          case 'P' :  // Neuplotten
          case 'R' :  // Rechteck fllen relativ
            nichtUnterst('R', s->aktuellesZeichen());
          break;
          default  :
            falsch('R', s->aktuellesZeichen());
        }
      break;
      case 'S' :
        s->liesEinZeichen();
        switch(s->aktuellesZeichen()) {
          case 'P' :  // Stift whlen
            s->liesEinZeichen();
            if(s->liesZahl(wert1)) {
              if(ignFalsch)
                s->ueberliesRest();
              else
                s->gibFehlerAus("SP: Farbnummer erwartet");
            }
            keinRestZul("SP");
            p->waehleFarbe(wert1);
          break;
          case 'A' :  // Alternativzeichensatz whlen
          case 'C' :  // Skalieren
          case 'I' :  // Absolute Zeichengrsse
          case 'L' :  // Schriftneigungswinkel
          case 'M' :  // Symbolmodus
          case 'R' :  // Relative Zeichengrsse
          case 'S' :  // Standardzeichensatz whlen
            nichtUnterst('S', s->aktuellesZeichen());
          break;
          default  :
            falsch('S', s->aktuellesZeichen());
        }
      break;
      case 'T' :
        s->liesEinZeichen();
        switch(s->aktuellesZeichen()) {
          case 'L' :  // Schrittlnge
            nichtUnterst('T', s->aktuellesZeichen());
          break;
          default  :
            falsch('T', s->aktuellesZeichen());
        }
      break;
      case 'U' :
        s->liesEinZeichen();
        switch(s->aktuellesZeichen()) {
          case 'C' :  // Benutzerdefiniertes Zeichen
          case 'F' :  // Benutzerdefinierte Fllart
            nichtUnterst('U', s->aktuellesZeichen());
          break;
          default  :
            falsch('U', s->aktuellesZeichen());
        }
      break;
      case 'V' :
        s->liesEinZeichen();
        switch(s->aktuellesZeichen()) {
          case 'S' :  // Stiftgeschwindigkeit
            s->ueberliesRest();  // immer ignorieren
          break;
          default  :
            falsch('V', s->aktuellesZeichen());
        }
      break;
      case 'W' :
        s->liesEinZeichen();
        switch(s->aktuellesZeichen()) {
          case 'D' :  // Auf Anzeige schreiben
          case 'G' :  // Keil fllen
            nichtUnterst('W', s->aktuellesZeichen());
          break;
          default  :
            falsch('W', s->aktuellesZeichen());
        }
      break;
      case 'X' :
        s->liesEinZeichen();
        switch(s->aktuellesZeichen()) {
          case 'T' :  // X-Schritt
            nichtUnterst('X', s->aktuellesZeichen());
          break;
          default  :
            falsch('X', s->aktuellesZeichen());
        }
      break;
      case 'Y' :
        s->liesEinZeichen();
        switch(s->aktuellesZeichen()) {
          case 'T' :  // Y-Schritt
            nichtUnterst('Y', s->aktuellesZeichen());
          break;
          default  :
            falsch('Y', s->aktuellesZeichen());
        }
      break;
      case ';' :  // Leeres Feld zulssig
        s->liesEinZeichen();
      break;
      default :
        falsch(s->aktuellesZeichen(), 'x');
    }
  }
  delete s;   // Datei durch Zerstrung des HPGL-Stromes wieder schliessen

  // Plot eingelesen => darstellen
  p->zeigGrafik(xAufl, yAufl, aFar);
  delete p;

  printf("HPGL-Anzeiger beendet.\n");
  return 0;
}
